#include <msp430x12x2.h>
#include "system.h"
#include "bits.h"
#include "nRF24L01.h"
#include "const.h"
#include "test.h"


unsigned char ch;
unsigned int i = 0;
unsigned char config, dummy, var, payload;
extern unsigned char status;
unsigned char status_temp;
unsigned char but1_flag = 0;

unsigned char but_intr_flag = 0;
unsigned char irq_intr_flag = 0;

volatile unsigned int dly;

#pragma vector=PORT2_VECTOR
__interrupt void handle_port_interrupt (void)
{
  // BUT
  if(P2IFG&0x08) {
    but_intr_flag = 1;
  }

  // IRQ
  if(P2IFG&0x10) {
    irq_intr_flag = 1;
  }

  // Clear all interupt flag
  P1IFG = 0x0;
  P2IFG = 0x0;

  // simple delay to avoid pulsation fo buttons
  for(dly=100; dly; dly--);

  // Exit from LPM3
  LPM3_EXIT;
}

unsigned char TestRTC(void)
{
  TACTL = 0x104;	// ACLK source, Issue RESET, No interrupts
  for(dly=10; dly; dly--);
  TACTL = 0x120; 	// ACLK source, continious mode;
  for(dly=100; dly; dly--);
  TACTL = 0x100;	// stop counting
  if ((TAR > 0x1c)&&(TAR<0x20)) return 0; // RTC OK
  	else return 1;
}

int main(void){

  // Frequency init
  FreqInit_LFXT1();
  FreqInit_DCO();


  // Test Ext port
  if (!(P2IN&BIT3)) 
 
  {
	
	if (TestRTC()) 
	{
	  P3DIR |= BIT7;
	  while (1)
	  {
		// Loop forever - rtc fault
		P3OUT |= BIT7;
	  	for(dly=20000; dly; dly--);
	  	P3OUT &= ~BIT7;	  
	  	for(dly=20000; dly; dly--);
	  }
	}
	P3OUT &= ~BIT7;	// Led On - RTC Pass
  	P3DIR |= BIT7;
	for(dly=65000; dly; dly--);
	while(TestEXT()==0);
	P3DIR |= BIT7;

	for (i=0;i<15;i++) 
	{
	  P3OUT |= BIT7;
	  for(dly=5000; dly; dly--);
	  P3OUT &= ~BIT7;	  
	  for(dly=5000; dly; dly--);
	}
  }

  // Light led if test pass
  P3OUT &= ~BIT7;
  P3DIR |= BIT7;
  
  //while(P2IN&BIT3);

  // System init
  //FreqInit_LFXT1();
  SysInit();

  // Init Interrupts
  InitInterrupts();

  // Init SPI0 Interface
  SPI0Init();

  // Setting for nRF chip
  nRFSetting();

  // Interrupt enable
  _EINT();

  // Claer variables
  i = 0;

  // loop forever
  while(1)
  {

    // BUTTON1 =================================================================
    // Button1 was pressed
    if(but_intr_flag)    {

      // Preinit
      SPI0Init();
      nRFSetting();

      // Disable enable
      _DINT();

      // Send one byte -> 0x45 -------------------------------------------------

      // Chip enable low
      CE_LOW();

      // Setting for TX device
      // Write CONFIG register -> 00001010 - CRC enable, power-up, TX
      status = SPI_Send_command_with_ADDR(W_REGISTER,CONFIG_REG_ADDR, 0x2A);

      // Send payload - send any data
      status = SPI_Send_command_without_ADDR(W_TX_PAYLOAD, 0x45);

      // Pulse for CE -> starts the transmission.
      CE_HIGH();
      CE_LOW();

      // Read STATUS register
      status = SPI_Send_command_without_ADDR(NOP, NOP);

      // if exceed number of transmision packets
      if ((status & MAX_RT) != 0) {

        // Clear MAX_RT bit in status register
        status_temp = SPI_Send_command_with_ADDR(W_REGISTER, STATUS_ADDR, (status|MAX_RT));

        // No communication event here
        // ....
      }

      // If packet sent on TX
      if ((status & TX_DS) != 0) {

        // Clear TX_DS bit in status register
        status_temp = SPI_Send_command_with_ADDR(W_REGISTER, STATUS_ADDR, (status|TX_DS));

        // Your code here
        // ...
      }


      // If TX full
      if ((status & TX_FULL) != 0)  {

        // Flush TX FIFO (in TX mode)
        status_temp = SPI_Send_command_without_ADDR(FLUSH_TX, NOP);

        // Your code here
        // ...
      }

      // End send one byte -> 0x45 ---------------------------------------------

      // Setting for RX device
      //Write CONFIG register -> 00001010 - CRC enable, power-up, RX
      status = SPI_Send_command_with_ADDR(W_REGISTER,CONFIG_REG_ADDR, 0x2B);

      // clear flag
      but_intr_flag=0;

      // Interrupt enable
      _EINT();

    }

    // Chip enable high
    CE_HIGH();

    // wait for receive signal - IRQ
    if (irq_intr_flag) {

      //Read STATUS status register
      status = SPI_Send_command_without_ADDR(NOP, NOP);

      // Set high when new data arrives RX FIFO
      if ((status & RX_DR) != 0) {

        // Chip enable low
        CE_LOW();

        //Read payload data
        payload=SPI_Send_command_without_ADDR(R_RX_PAYLOAD, NOP);

        // Clear RX_DR bit in status register
        status_temp = SPI_Send_command_with_ADDR(W_REGISTER, STATUS_ADDR, (status|RX_DR));

        // dummy data
        SPI_Send_command_without_ADDR(R_RX_PAYLOAD, NOP);
        SPI_Send_command_without_ADDR(R_RX_PAYLOAD, NOP);
        SPI_Send_command_without_ADDR(R_RX_PAYLOAD, NOP);
        SPI_Send_command_without_ADDR(R_RX_PAYLOAD, NOP);

      }

      // Check received data
      if (payload == 0x45)  {

        // Clear data
        payload = 0;

        // Do anything - turn on/off led
        if ((P3IN & BIT7) != 0) {
          LED1_ON;
        }
        else {
          LED1_OFF;
        }
      }

      // Clear flag
      irq_intr_flag = 0;
    }

    // Chip enable high
    CE_HIGH();

    // Go to low power mode 3
    LPM3;

  }
}


